local addon, FM = ...

local default_settings = {
	mmx=350, mmy=350, locked = false, visible=true, size=32,
	items = {}
}

local function MergeTable(o,n)
	for k,v in pairs(n) do
		if type(v) == "table" then
			if o[k] == nil then
				o[k] = {}			
			end
	 	 	if type(o[k]) == 'table' then
	 			MergeTable(o[k], n[k])
	 	 	end
		else
			if o[k] == nil then
				o[k] = v
			end
		end
	end
end

-- /script Inspect.Addon.Detail("Farmer").data.UI.btn.MouseDown = false

function FM.HandleButton()
	if Farmer_Settings.locked then
		if Farmer_Settings.visible then
			FM.UI.btn:SetTexture(addon.identifier, "img/btn_green.png")
			FM.UI.ctr:SetVisible(true)
		else
			FM.UI.btn:SetTexture(addon.identifier, "img/btn_red.png")
			FM.UI.ctr:SetVisible(false)
		end
	else
		FM.UI.btn:SetTexture(addon.identifier, "img/btn_gray.png")
		FM.UI.ctr:SetVisible(false)
	end			
end

function FM.Resize()
	local sz = Farmer_Settings.size
	local db = sz*2
	local hf = math.floor(sz * 0.5)
	
	for k,v in pairs(FM.UI.icons) do
		v.f:SetWidth(sz)
		v.f:SetHeight(sz)
		v.t:SetFontSize(hf)
	end
	
	FM.UI.btn:SetWidth(db)
	FM.UI.btn:SetHeight(db)
end

function FM.ItemFrame(i, it)
	local f = UI.CreateFrame("Texture", "FM.UI.icons.f:"..i, FM.UI.ctr)
	f:SetLayer(5)
	f:SetTexture("Rift", i)
	f:SetWidth(Farmer_Settings.size)
	f:SetHeight(Farmer_Settings.size)
	--f.it = it
	
	f:EventAttach(Event.UI.Input.Mouse.Right.Click, function(self, h)
		Farmer_Settings.items[it] = nil
		f:SetVisible(false)
		FM.ArrangeIcons()
	end, "Event.UI.Input.Mouse.Right.Click")
	
	local t = UI.CreateFrame("Text", "FM.UI.icons.t:"..i, f)
	t:SetFontSize(math.floor(Farmer_Settings.size * 0.5))
	t:SetPoint("CENTER", f, "CENTER")
	t:SetLayer(10)
	t:SetEffectGlow({strength=5, blurX=2, blurY=2})
	
	return {f=f, t=t}
end

function FM.ArrangeIcons()
	local itab = {}
	for k,v in pairs(Farmer_Settings.items) do
		table.insert(itab, {n=v.n, t=k})
	end
	table.sort(itab, function(a,b) return a.n < b.n end)
	
	local rows = 0
	local cols = 0
	
	for x=1, #itab do
		local it = itab[x].t
		FM.UI.icons[it].f:SetVisible(true)
		if x==1 then
			FM.UI.icons[it].f:SetPoint("TOPLEFT", FM.UI.ctr, "TOPLEFT", 0,0)
		elseif x%10 == 1 then
			local pf = itab[x-10].t
			FM.UI.icons[it].f:SetPoint("TOPLEFT", FM.UI.icons[pf].f, "TOPRIGHT", 0,0)
		else
			local pf = itab[x-1].t
			FM.UI.icons[it].f:SetPoint("TOPLEFT", FM.UI.icons[pf].f, "BOTTOMLEFT", 0,0)
		end
	end
	
	cols = math.ceil(#itab/10)
	if cols > 1 then
		rows = 10
	else
		rows = #itab
	end
	
	FM.UI.ctr:SetHeight(rows * Farmer_Settings.size)
	FM.UI.ctr:SetHeight(cols * Farmer_Settings.size)
	
end

function FM.BuildUI()
	FM.UI = {}
	FM.UI.icons = {}
	
	FM.UI.context = UI.CreateContext(addon.identifier)
	FM.UI.btn = UI.CreateFrame("Texture", "FM.UI.btn", FM.UI.context)
		
	FM.UI.btn:EventAttach(Event.UI.Input.Mouse.Right.Click, function(self, h)
		if not self.MouseDown then
			Farmer_Settings.locked = not Farmer_Settings.locked
			FM.HandleButton()
		end
	end, "Event.UI.Input.Mouse.Right.Click")
	
	FM.UI.btn:EventAttach(Event.UI.Input.Mouse.Left.Click, function(self, h)
		if Farmer_Settings.locked then
			Farmer_Settings.visible = not Farmer_Settings.visible
			FM.HandleButton()
		end
	end, "Event.UI.Input.Mouse.Left.Click")

	FM.UI.btn:EventAttach(Event.UI.Input.Mouse.Left.Up, function(self, h)
		if Farmer_Settings.locked then
			local a,b = Inspect.Cursor()
			if a and b then
				if a == "item" or a == "itemtype" then
					local id = Inspect.Item.Detail(b)
					if Farmer_Settings.items[id.type] == nil then
						Farmer_Settings.items[id.type] = {
							n = id.name,
							c = 0,
							i = id.icon
						}
						if FM.UI.icons[id.type] == nil then
							FM.UI.icons[id.type] = FM.ItemFrame(id.icon, id.type)
						end
						FM.ArrangeIcons()
						Library.LibInventory.RegisterItem(id.type, FM.cb_libInvItems)
					else
						print(string.format("%s already being watched.", id.name))
					end
				end
			end
			Command.Cursor(nil)
		else
			self.MouseDown = false
			Farmer_Settings.mmx = FM.UI.btn:GetLeft()
			Farmer_Settings.mmy = FM.UI.btn:GetTop()
		end
	end, "Event.UI.Input.Mouse.Left.Up")
	
	FM.UI.btn:EventAttach(Event.UI.Input.Mouse.Left.Down, function(self, h)
		if not Farmer_Settings.locked then
			self.MouseDown = true
			local mouseData = Inspect.Mouse()
			self.sx = mouseData.x - FM.UI.btn:GetLeft()
			self.sy = mouseData.y - FM.UI.btn:GetTop()
		end
	end, "Event.UI.Input.Mouse.Left.Down")

	FM.UI.btn:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function(self, h)
		if self.MouseDown then
			local nx, ny
			local mouseData = Inspect.Mouse()
			nx = mouseData.x - self.sx
			ny = mouseData.y - self.sy
			FM.UI.btn:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nx,ny)			
		end
	end, "Event.UI.Input.Mouse.Cursor.Move")

	FM.UI.btn:EventAttach(Event.UI.Input.Mouse.Wheel.Back, function(self, h)
		if Farmer_Settings.size > 12 then
			Farmer_Settings.size = Farmer_Settings.size-1
			FM.Resize()
		end
	end, "Event.UI.Input.Mouse.Wheel.Back")
	
	FM.UI.btn:EventAttach(Event.UI.Input.Mouse.Wheel.Forward, function(self, h)
		if Farmer_Settings.size < 48 then
			Farmer_Settings.size = Farmer_Settings.size+1
			FM.Resize()
		end
	end, "Event.UI.Input.Mouse.Wheel.Forward")
	
	FM.UI.ctr = UI.CreateFrame("Frame", "FM.UI.ctr", FM.UI.context)
	FM.UI.ctr:SetPoint("TOPLEFT", FM.UI.btn, "BOTTOMLEFT", 0,0)
	FM.UI.ctr:SetLayer(0)
end

function FM.cb_libInvItems(t)
	local cnt = 0
	for k,v in pairs(t) do
		cnt = 0
		if v["PLAYER"] then
			for ik, iv in pairs(v["PLAYER"]) do
				if Library.LibInventory.REFSLOTS["inventory"][ik] then
					cnt = cnt+iv
				end
			end
		end
		FM.UI.icons[k].t:SetText(tostring(cnt))
	end
end

function FM.Event_Unit_Availability_Full(h,t)
	for k,v in pairs(t) do
		if v == "player" then
			Library.LibInventory.RegisterItem(Farmer_Settings.items, FM.cb_libInvItems)
			Command.Event.Detach(Event.Unit.Availability.Full, nil, nil, nil, addon.identifier)
		end
	end
end

function FM.Event_Addon_SavedVariables_Load_End(h,a)
	if a == addon.identifier then
		if Farmer_Settings == nil then Farmer_Settings = {} end
		MergeTable(Farmer_Settings, default_settings)
		
		FM.UI.btn:SetPoint("TOPLEFT", UIParent, "TOPLEFT", Farmer_Settings.mmx, Farmer_Settings.mmy)
		FM.UI.btn:SetHeight(Farmer_Settings.size *2)
		FM.UI.btn:SetWidth(Farmer_Settings.size *2)
		FM.HandleButton()
		
		for k,v in pairs(Farmer_Settings.items) do
			FM.UI.icons[k] = FM.ItemFrame(v.i, k)
		end
		FM.ArrangeIcons()
	end
end

FM.BuildUI()

Command.Event.Attach(Event.Unit.Availability.Full, FM.Event_Unit_Availability_Full, "Event.Unit.Availability.Full")
Command.Event.Attach(Event.Addon.SavedVariables.Load.End, FM.Event_Addon_SavedVariables_Load_End, "Event.Addon.SavedVariables.Load.End")

print(string.format("v%s loaded.", addon.toc.Version))
